import 'package:path/path.dart' as path;
import 'package:pull_request_coverage/src/presentation/logger/log_level.dart';
import 'package:pull_request_coverage/src/presentation/logger/logger.dart';

/// [GetUncoveredFileLines] returns a list of line index that are not covered by tests.
/// returns `null` if the file doesn't exist in lcov.info file
class GetUncoveredFileLines {
  bool _isSamePath(String filePathFromDiff, String lcovFileHeader) {
    if (!lcovFileHeader.startsWith("SF:")) {
      // not a lcov header
      return false;
    }

    final String filePathFromLcov;
    filePathFromDiff = path.normalize(filePathFromDiff);

    {
      final indexOfLib = lcovFileHeader.indexOf("/lib/");
      if (lcovFileHeader.startsWith("SF:/") && indexOfLib >= 0) {
        // lcov.info generated by `coverage` package uses absolute paths
        // we need to transform it to relative
        filePathFromLcov =
            path.normalize(lcovFileHeader.substring(indexOfLib + 1));
      } else {
        // lcov.info generated by `flutter test --coverage` uses relative path already.
        // We only need to remove `SF:` prefix and normalize it
        filePathFromLcov = path.normalize(lcovFileHeader.substring(3));
      }
    }
    return filePathFromLcov == filePathFromDiff;
  }

  List<int>? call(List<String> lcovInfoLines, String filePath) {
    for (var i = 0; i < lcovInfoLines.length; i++) {
      final line = lcovInfoLines[i];
      if (_isSamePath(filePath, line)) {
        final uncoveredLines = <int>[];
        for (var j = i + 1; j < lcovInfoLines.length; j++) {
          final line = lcovInfoLines[j];
          if (line.startsWith("end_of_record")) {
            return uncoveredLines;
          }
          if (line.startsWith("DA:")) {
            final lineInfo = line.substring(line.indexOf(":") + 1).split(",");
            final lineNumber = int.parse(lineInfo[0]);
            final covered = lineInfo[1] != "0";
            if (!covered) {
              uncoveredLines.add(lineNumber);
            }
          }
        }
      }
    }
    logger.log(
      tag: "GetUncoveredFileLines",
      message: "coverage info of `$filePath` not found in lcov.info",
      level: LogLevel.verbose,
    );
    return null;
  }
}
